% RDK Matlab Collect and Plot
% Connects to RDK, Reads default parameters from RDK, Collects data, Plots data
% Note: Requires NI-VISA Runtime Engine to be installed

clear
clc
instrreset

% Connect to RDK
serialNumber = input('Enter RDK Serial Number: ');
visaObjectString = sprintf('USB0::0x2012::0x0017::%04d::INSTR',serialNumber);
visaObject = visa('ni',visaObjectString);
disp('Connecting to RDK...')
fopen(visaObject); pause(0.5)

% Query RDK for parameters
disp('Querying RDK Parameters...')
fprintf(visaObject,'SWEEP:FREQSTAR?\n');
startFrequency = str2double(fscanf(visaObject));
fprintf(visaObject,'SWEEP:FREQSTOP?\n');
stopFrequency = str2double(fscanf(visaObject));
fprintf(visaObject,'SWEEP:RAMPTIME?\n');
rampTime = str2double(fscanf(visaObject));

% Prepare Data Capture
numSamples = 1024;
samplesPerRead = 31;
numReads = ceil(numSamples/samplesPerRead);
hexSamples = char(zeros(1,numSamples*4+1));
sampleRate = 50e-6;
sampleFreq = 1/sampleRate;

% Request Data Capture
disp('Requesting Data Capture...')
fprintf(visaObject,['CAPT:FRAM ',num2str(numSamples)]);

% Collect Data Capture
disp('Collecting Data...')
for n = 1:numReads
    fprintf(visaObject,'CAPT:FRAM?\n');
    hexString = fscanf(visaObject);
    for index = 1:length(hexString)-1
        hexSamples((n-1)*samplesPerRead*4+index) = hexString(index);
    end
end

% Close RDK
disp('Disconnecting from RDK...')
fclose(visaObject);

% Format Data
disp('Formatting Data...')
reshapedHex = reshape(hexSamples(1:end-1),4,[]).';
unsignedSamples = hex2dec(reshapedHex);
samples = unsignedSamples/65535*5-2.5;

disp(sprintf('%d samples collected. Plotting...',length(samples)))

% Raw Data Plot
figure(1)
timeAxis = 0:sampleRate:(length(samples)-1)*sampleRate;
plot(timeAxis,samples)
xlabel('Time (s)')
ylabel('Amplitude (V)')
title('Raw Data')
axis([0,max(timeAxis),-2.5,2.5])
grid on

% Beat Frequency Spectrum Plot
figure(2)
nfft = length(samples)*7;
samplesFFT = fft(samples,nfft)/nfft;
samplesFFTdBm = mag2db(abs(samplesFFT));
freqAxis = sampleFreq/2*linspace(0,1,nfft/2+1);
plot(freqAxis,samplesFFTdBm(1:nfft/2+1))
xlabel('Frequency (Hz)')
ylabel('Power (dBm)')
title('Beat Frequency Spectrum')
axis([0,max(freqAxis),min(samplesFFTdBm)-20,max(samplesFFTdBm)+20])
grid on

% Range Plot
figure(3)
rangeAxis = (3e8*((0:nfft/2)*sampleFreq/nfft))/(2*(1e9*(stopFrequency-startFrequency)/(rampTime/1000)));
plot(rangeAxis,samplesFFTdBm(1:nfft/2+1))
xlabel('Range (m)')
ylabel('Power (dBm)')
title('Range')
axis([0,max(rangeAxis),min(samplesFFTdBm)-20,max(samplesFFTdBm)+20])
grid on